"""RtiTek TRV devices support."""
import logging
from typing import Optional, Union

import zigpy.types as t
from zhaquirks import Bus, LocalDataCluster
from zhaquirks.const import (
    DEVICE_TYPE,
    ENDPOINTS,
    INPUT_CLUSTERS,
    MODELS_INFO,
    OUTPUT_CLUSTERS,
    PROFILE_ID,
)
from zhaquirks.tuya import (
    TuyaManufClusterAttributes,
    TuyaPowerConfigurationCluster,
    TuyaThermostat,
    TuyaThermostatCluster,
    TuyaUserInterfaceCluster,
)
from zigpy.profiles import zha
from zigpy.zcl import foundation
from zigpy.zcl.clusters.general import (
    AnalogOutput,
    Basic,
    BinaryInput,
    Groups,
    OnOff,
    Ota,
    Scenes,
    Time,
)
from zigpy.zcl.clusters.hvac import Thermostat

_LOGGER = logging.getLogger(__name__)

RTI_TARGET_TEMP_ATTR = 0x0202  # target room temp (decidegree)
RTI_TEMPERATURE_ATTR = 0x0203  # current room temp (decidegree)
RTI_MODE_ATTR = 0x0401  # [0] schedule [1] manual [2] off [3] on
RTI_CHILD_LOCK_ATTR = 0x010C  # [0] unlocked [1] locked
RTI_TEMP_CALIBRATION_ATTR = 0x0265  # temperature calibration (degree)
RTI_MIN_TEMPERATURE_ATTR = 0x020F  # minimum limit of temperature setting (decidegree)
RTI_MAX_TEMPERATURE_ATTR = 0x0210  # maximum limit of temperature setting (decidegree)
RTI_WINDOW_DETECT_ATTR = 0x0108  # [0] alarm not active [1] alarm active
RTI_BOOST_ATTR = 0x0104  # [0] off [1] on
RTI_BOOST_COUNTDOWN_ATTR = 0x0205  # (seconds)
RTI_VALVE_POSITION_ATTR = 0x0266  # opening percentage /10
RTI_VALVE_STATE_ATTR = 0x0406  # [0] closed [1] opened
RTI_BATTERY_ATTR = 0x020D  # battery percentage remaining 0-100%
BECA_SCHEDULE = 0x1111  # schedule
# [6, 0, 40, 11, 30, 42, 13, 30, 44, 17, 30, 46, 6, 0, 48, 12, 0, 46, 14, 30, 44, 17, 30, 42, 6, 0, 38, 12, 30, 40, 14, 30, 42, 18, 30, 40] for :
# Monday to friday : 06:00 20°C / 11:30 21°C / 13:30 22°C / 17:30 23°C
# Saturday : 06:00 24°C / 12:00 23°C / 14:30 22°C / 17:30 21°C
# Sunday : 06:00 19°C / 12:30 20°C / 14:30 21°C / 18:30 20°C )
RtiManufClusterSelf = {}


class data288(t.FixedList, item_type=t.uint8_t, length=36):
    """General data, Discrete, 288 bit."""

    pass


class CustomTuyaOnOff(LocalDataCluster, OnOff):
    """Custom Tuya OnOff cluster."""

    def __init__(self, *args, **kwargs):
        """Init."""
        super().__init__(*args, **kwargs)
        self.endpoint.device.thermostat_onoff_bus.add_listener(self)

    # pylint: disable=R0201
    def map_attribute(self, attribute, value):
        """Map standardized attribute value to dict of manufacturer values."""
        return {}

    async def write_attributes(self, attributes, manufacturer=None):
        """Implement writeable attributes."""

        records = self._write_attr_records(attributes)

        if not records:
            return [[foundation.WriteAttributesStatusRecord(foundation.Status.SUCCESS)]]

        manufacturer_attrs = {}
        for record in records:
            attr_name = self.attributes[record.attrid][0]
            new_attrs = self.map_attribute(attr_name, record.value.value)

            _LOGGER.debug(
                "[0x%04x:%s:0x%04x] Mapping standard %s (0x%04x) "
                "with value %s to custom %s",
                self.endpoint.device.nwk,
                self.endpoint.endpoint_id,
                self.cluster_id,
                attr_name,
                record.attrid,
                repr(record.value.value),
                repr(new_attrs),
            )

            manufacturer_attrs.update(new_attrs)

        if not manufacturer_attrs:
            return [
                [
                    foundation.WriteAttributesStatusRecord(
                        foundation.Status.FAILURE, r.attrid
                    )
                    for r in records
                ]
            ]

        await RtiManufClusterSelf[
            self.endpoint.device.ieee
        ].endpoint.tuya_manufacturer.write_attributes(
            manufacturer_attrs, manufacturer=manufacturer
        )

        return [[foundation.WriteAttributesStatusRecord(foundation.Status.SUCCESS)]]

    async def command(
        self,
        command_id: Union[foundation.Command, int, t.uint8_t],
        *args,
        manufacturer: Optional[Union[int, t.uint16_t]] = None,
        expect_reply: bool = True,
        tsn: Optional[Union[int, t.uint8_t]] = None,
    ):
        """Override the default Cluster command."""

        if command_id in (0x0000, 0x0001, 0x0002):

            if command_id == 0x0000:
                value = False
            elif command_id == 0x0001:
                value = True
            else:
                attrid = self.attridx["on_off"]
                success, _ = await self.read_attributes(
                    (attrid,), manufacturer=manufacturer
                )
                try:
                    value = success[attrid]
                except KeyError:
                    return foundation.Status.FAILURE
                value = not value

            (res,) = await self.write_attributes(
                {"on_off": value},
                manufacturer=manufacturer,
            )
            return [command_id, res[0].status]

        return [command_id, foundation.Status.UNSUP_CLUSTER_COMMAND]


class RtiManufCluster(TuyaManufClusterAttributes):
    """Manufacturer Specific Cluster of thermostatic valves."""

    def __init__(self, *args, **kwargs):
        """Init."""
        super().__init__(*args, **kwargs)
        global RtiManufClusterSelf
        RtiManufClusterSelf[self.endpoint.device.ieee] = self

    set_time_offset = 1970

    manufacturer_attributes = {
        RTI_TEMPERATURE_ATTR: ("temperature", t.uint32_t),
        RTI_TARGET_TEMP_ATTR: ("target_temperature", t.uint32_t),
        RTI_MODE_ATTR: ("mode", t.uint8_t),
        RTI_CHILD_LOCK_ATTR: ("child_lock", t.uint8_t),
        RTI_TEMP_CALIBRATION_ATTR: ("temperature_calibration", t.int32s),
        RTI_MIN_TEMPERATURE_ATTR: ("min_temperature", t.uint32_t),
        RTI_MAX_TEMPERATURE_ATTR: ("max_temperature", t.uint32_t),
        RTI_WINDOW_DETECT_ATTR: ("window_detection", t.uint8_t),
        RTI_BOOST_ATTR: ("boost_enabled", t.uint8_t),
        RTI_BOOST_COUNTDOWN_ATTR: ("boost_duration_seconds", t.uint32_t),
        RTI_VALVE_POSITION_ATTR: ("valve_position", t.uint32_t),
        RTI_VALVE_STATE_ATTR: ("valve_state", t.uint8_t),
        RTI_BATTERY_ATTR: ("battery", t.uint32_t),
        BECA_SCHEDULE: ("schedule", data288),
    }

    DIRECT_MAPPED_ATTRS = {
        RTI_TEMPERATURE_ATTR: ("local_temp", lambda value: value * 10),
        RTI_TARGET_TEMP_ATTR: ("occupied_heating_setpoint", lambda value: value * 10),
        RTI_TEMP_CALIBRATION_ATTR: ("local_temperature_calibration", None),
        RTI_MIN_TEMPERATURE_ATTR: (
            "min_heat_setpoint_limit",
            lambda value: value * 10,
        ),
        RTI_MAX_TEMPERATURE_ATTR: (
            "max_heat_setpoint_limit",
            lambda value: value * 10,
        ),
    }

    def _update_attribute(self, attrid, value):
        """Override default _update_attribute."""
        super()._update_attribute(attrid, value)
        if attrid in self.DIRECT_MAPPED_ATTRS:
            self.endpoint.device.thermostat_bus.listener_event(
                "temperature_change",
                self.DIRECT_MAPPED_ATTRS[attrid][0],
                value
                if self.DIRECT_MAPPED_ATTRS[attrid][1] is None
                else self.DIRECT_MAPPED_ATTRS[attrid][1](value),
            )

        if attrid == RTI_WINDOW_DETECT_ATTR:
            self.endpoint.device.RtiWindowDetection_bus.listener_event(
                "set_value", value
            )
        elif attrid == RTI_CHILD_LOCK_ATTR:
            mode = 1 if value else 0
            self.endpoint.device.ui_bus.listener_event("child_lock_change", mode)
            self.endpoint.device.thermostat_onoff_bus.listener_event(
                "child_lock_change", mode
            )
        elif attrid in (RTI_MODE_ATTR, RTI_BOOST_ATTR):
            if attrid == RTI_BOOST_ATTR and value == 1:
                self.endpoint.device.thermostat_bus.listener_event("mode_change", 5)
            elif attrid == RTI_MODE_ATTR:
                self.endpoint.device.thermostat_bus.listener_event("mode_change", value)
        elif attrid == RTI_VALVE_POSITION_ATTR:
            self.endpoint.device.RtiValvePosition_bus.listener_event(
                "set_value", round(value / 10)
            )
        elif attrid == RTI_VALVE_STATE_ATTR:
            self.endpoint.device.thermostat_bus.listener_event("state_change", value)
        elif attrid == RTI_TEMP_CALIBRATION_ATTR:
            self.endpoint.device.RtiTempCalibration_bus.listener_event(
                "set_value", value
            )
        elif attrid == RTI_BOOST_COUNTDOWN_ATTR:
            self.endpoint.device.RtiBoostCountdown_bus.listener_event(
                "set_value", value
            )
        elif attrid == RTI_BATTERY_ATTR:
            self.endpoint.device.battery_bus.listener_event("battery_change", value)
        elif attrid == RTI_MIN_TEMPERATURE_ATTR:
            self.endpoint.device.RtiMinTemp_bus.listener_event("set_value", value)
        elif attrid == RTI_MAX_TEMPERATURE_ATTR:
            self.endpoint.device.RtiMaxTemp_bus.listener_event("set_value", value)
        elif attrid == BECA_SCHEDULE:
            self.endpoint.device.thermostat_bus.listener_event("schedule_change", value)


class RtiThermostat(TuyaThermostatCluster):
    """Thermostat cluster for thermostatic valves."""

    class Preset(t.enum8):
        """Working modes of the thermostat."""

        Away = 0x00
        Schedule = 0x01
        Manual = 0x02
        Comfort = 0x03
        Eco = 0x04
        Boost = 0x05
        Complex = 0x06
        TempManual = 0x07

    class WorkDays(t.enum8):
        """Workday configuration for scheduler operation mode."""

        MonToFri = 0x00
        MonToSat = 0x01
        MonToSun = 0x02

    class ForceValveState(t.enum8):
        """Force valve state option."""

        Normal = 0x00
        Open = 0x01
        Close = 0x02

    _CONSTANT_ATTRIBUTES = {
        0x001B: Thermostat.ControlSequenceOfOperation.Heating_Only,
        0x001C: Thermostat.SystemMode.Heat,
    }

    manufacturer_attributes = {
        0x4002: ("operation_preset", Preset),
        0x4110: ("schedule_workday_1_hour", t.uint8_t),
        0x4111: ("schedule_workday_1_minute", t.uint8_t),
        0x4112: ("schedule_workday_1_temperature", t.uint8_t),
        0x4113: ("schedule_workday_2_hour", t.uint8_t),
        0x4114: ("schedule_workday_2_minute", t.uint8_t),
        0x4115: ("schedule_workday_2_temperature", t.uint8_t),
        0x4116: ("schedule_workday_3_hour", t.uint8_t),
        0x4117: ("schedule_workday_3_minute", t.uint8_t),
        0x4118: ("schedule_workday_3_temperature", t.uint8_t),
        0x4119: ("schedule_workday_4_hour", t.uint8_t),
        0x4120: ("schedule_workday_4_minute", t.uint8_t),
        0x4121: ("schedule_workday_4_temperature", t.uint8_t),
        0x4122: ("schedule_saturday_1_hour", t.uint8_t),
        0x4123: ("schedule_saturday_1_minute", t.uint8_t),
        0x4124: ("schedule_saturday_1_temperature", t.uint8_t),
        0x4125: ("schedule_saturday_2_hour", t.uint8_t),
        0x4126: ("schedule_saturday_2_minute", t.uint8_t),
        0x4127: ("schedule_saturday_2_temperature", t.uint8_t),
        0x4128: ("schedule_saturday_3_hour", t.uint8_t),
        0x4129: ("schedule_saturday_3_minute", t.uint8_t),
        0x4130: ("schedule_saturday_3_temperature", t.uint8_t),
        0x4131: ("schedule_saturday_4_hour", t.uint8_t),
        0x4132: ("schedule_saturday_4_minute", t.uint8_t),
        0x4133: ("schedule_saturday_4_temperature", t.uint8_t),
        0x4134: ("schedule_sunday_1_hour", t.uint8_t),
        0x4135: ("schedule_sunday_1_minute", t.uint8_t),
        0x4136: ("schedule_sunday_1_temperature", t.uint8_t),
        0x4137: ("schedule_sunday_2_hour", t.uint8_t),
        0x4138: ("schedule_sunday_2_minute", t.uint8_t),
        0x4139: ("schedule_sunday_2_temperature", t.uint8_t),
        0x4140: ("schedule_sunday_3_hour", t.uint8_t),
        0x4141: ("schedule_sunday_3_minute", t.uint8_t),
        0x4142: ("schedule_sunday_3_temperature", t.uint8_t),
        0x4143: ("schedule_sunday_4_hour", t.uint8_t),
        0x4144: ("schedule_sunday_4_minute", t.uint8_t),
        0x4145: ("schedule_sunday_4_temperature", t.uint8_t),
    }

    DIRECT_MAPPING_ATTRS = {
        "min_heat_setpoint_limit": (
            RTI_MIN_TEMPERATURE_ATTR,
            lambda value: round(value / 10),
        ),
        "max_heat_setpoint_limit": (
            RTI_MAX_TEMPERATURE_ATTR,
            lambda value: round(value / 10),
        ),
        "local_temperature_calibration": (
            RTI_TEMP_CALIBRATION_ATTR,
            lambda value: value,
        ),
    }

    SCHEDULE_ATTRS = {
        "schedule_sunday_4_temperature": 20,
        "schedule_sunday_4_minute": 30,
        "schedule_sunday_4_hour": 18,
        "schedule_sunday_3_temperature": 21,
        "schedule_sunday_3_minute": 30,
        "schedule_sunday_3_hour": 14,
        "schedule_sunday_2_temperature": 20,
        "schedule_sunday_2_minute": 30,
        "schedule_sunday_2_hour": 12,
        "schedule_sunday_1_temperature": 19,
        "schedule_sunday_1_minute": 0,
        "schedule_sunday_1_hour": 6,
        "schedule_saturday_4_temperature": 21,
        "schedule_saturday_4_minute": 30,
        "schedule_saturday_4_hour": 17,
        "schedule_saturday_3_temperature": 22,
        "schedule_saturday_3_minute": 30,
        "schedule_saturday_3_hour": 14,
        "schedule_saturday_2_temperature": 23,
        "schedule_saturday_2_minute": 00,
        "schedule_saturday_2_hour": 12,
        "schedule_saturday_1_temperature": 24,
        "schedule_saturday_1_minute": 0,
        "schedule_saturday_1_hour": 6,
        "schedule_workday_4_temperature": 23,
        "schedule_workday_4_minute": 30,
        "schedule_workday_4_hour": 17,
        "schedule_workday_3_temperature": 22,
        "schedule_workday_3_minute": 30,
        "schedule_workday_3_hour": 13,
        "schedule_workday_2_temperature": 21,
        "schedule_workday_2_minute": 30,
        "schedule_workday_2_hour": 11,
        "schedule_workday_1_temperature": 20,
        "schedule_workday_1_minute": 0,
        "schedule_workday_1_hour": 6,
    }

    def map_attribute(self, attribute, value):
        """Map standardized attribute value to dict of manufacturer values."""

        if attribute in self.DIRECT_MAPPING_ATTRS:
            return {
                self.DIRECT_MAPPING_ATTRS[attribute][0]: value
                if self.DIRECT_MAPPING_ATTRS[attribute][1] is None
                else self.DIRECT_MAPPING_ATTRS[attribute][1](value)
            }

        if attribute == "occupied_heating_setpoint":
            return {RTI_TARGET_TEMP_ATTR: value / 10}

        if attribute == "operation_preset":
            if value == 1:
                return {RTI_MODE_ATTR: 0, RTI_BOOST_ATTR: 0}
            if value == 2:
                return {RTI_MODE_ATTR: 1, RTI_BOOST_ATTR: 0}
            if value == 5:
                return {RTI_BOOST_ATTR: 1}

        if attribute in ("programing_oper_mode", "occupancy"):
            if attribute == "occupancy":
                occupancy = value
                oper_mode = self._attr_cache.get(
                    self.attridx["programing_oper_mode"],
                    self.ProgrammingOperationMode.Simple,
                )
            else:
                occupancy = self._attr_cache.get(
                    self.attridx["occupancy"], self.Occupancy.Occupied
                )
                oper_mode = value
            if occupancy == self.Occupancy.Occupied:
                if oper_mode == self.ProgrammingOperationMode.Schedule_programming_mode:
                    return {RTI_MODE_ATTR: 0}
                if oper_mode == self.ProgrammingOperationMode.Simple:
                    return {RTI_MODE_ATTR: 1}
                self.error("Unsupported value for ProgrammingOperationMode")
            else:
                self.error("Unsupported value for Occupancy")

        if attribute in self.SCHEDULE_ATTRS:
            data = data288()
            for num, (attr, default) in enumerate(self.SCHEDULE_ATTRS.items()):
                if num % 3 == 0:
                    if attr == attribute:
                        val = round(value * 2)
                    else:
                        val = round(
                            self._attr_cache.get(self.attridx[attr], default) * 2
                        )
                else:
                    if attr == attribute:
                        val = value
                    else:
                        val = self._attr_cache.get(self.attridx[attr], default)

                data.append(val)
            return {BECA_SCHEDULE: data}

    def mode_change(self, value):
        """System Mode change."""
        if value == 1:
            operation_preset = self.Preset.Manual
            prog_mode = self.ProgrammingOperationMode.Simple
            occupancy = self.Occupancy.Occupied
        elif value == 5:
            operation_preset = self.Preset.Boost
            prog_mode = self.ProgrammingOperationMode.Simple
            occupancy = self.Occupancy.Occupied
        else:
            operation_preset = self.Preset.Schedule
            prog_mode = self.ProgrammingOperationMode.Schedule_programming_mode
            occupancy = self.Occupancy.Occupied

        self._update_attribute(self.attridx["programing_oper_mode"], prog_mode)
        self._update_attribute(self.attridx["occupancy"], occupancy)
        self._update_attribute(self.attridx["operation_preset"], operation_preset)

    def schedule_change(self, value):
        """Scheduler attribute change."""
        self._update_attribute(self.attridx["schedule_workday_1_hour"], value[35])
        self._update_attribute(self.attridx["schedule_workday_1_minute"], value[34])
        self._update_attribute(
            self.attridx["schedule_workday_1_temperature"], value[33] / 2
        )
        self._update_attribute(self.attridx["schedule_workday_2_hour"], value[32])
        self._update_attribute(self.attridx["schedule_workday_2_minute"], value[31])
        self._update_attribute(
            self.attridx["schedule_workday_2_temperature"], value[30] / 2
        )
        self._update_attribute(self.attridx["schedule_workday_3_hour"], value[29])
        self._update_attribute(self.attridx["schedule_workday_3_minute"], value[28])
        self._update_attribute(
            self.attridx["schedule_workday_3_temperature"], value[27] / 2
        )
        self._update_attribute(self.attridx["schedule_workday_4_hour"], value[26])
        self._update_attribute(self.attridx["schedule_workday_4_minute"], value[25])
        self._update_attribute(
            self.attridx["schedule_workday_4_temperature"], value[24] / 2
        )
        self._update_attribute(self.attridx["schedule_saturday_1_hour"], value[23])
        self._update_attribute(self.attridx["schedule_saturday_1_minute"], value[22])
        self._update_attribute(
            self.attridx["schedule_saturday_1_temperature"], value[21] / 2
        )
        self._update_attribute(self.attridx["schedule_saturday_2_hour"], value[20])
        self._update_attribute(self.attridx["schedule_saturday_2_minute"], value[19])
        self._update_attribute(
            self.attridx["schedule_saturday_2_temperature"], value[18] / 2
        )
        self._update_attribute(self.attridx["schedule_saturday_3_hour"], value[17])
        self._update_attribute(self.attridx["schedule_saturday_3_minute"], value[16])
        self._update_attribute(
            self.attridx["schedule_saturday_3_temperature"], value[15] / 2
        )
        self._update_attribute(self.attridx["schedule_saturday_4_hour"], value[14])
        self._update_attribute(self.attridx["schedule_saturday_4_minute"], value[13])
        self._update_attribute(
            self.attridx["schedule_saturday_4_temperature"], value[12] / 2
        )
        self._update_attribute(self.attridx["schedule_sunday_1_hour"], value[11])
        self._update_attribute(self.attridx["schedule_sunday_1_minute"], value[10])
        self._update_attribute(
            self.attridx["schedule_sunday_1_temperature"], value[9] / 2
        )
        self._update_attribute(self.attridx["schedule_sunday_2_hour"], value[8])
        self._update_attribute(self.attridx["schedule_sunday_2_minute"], value[7])
        self._update_attribute(
            self.attridx["schedule_sunday_2_temperature"], value[6] / 2
        )
        self._update_attribute(self.attridx["schedule_sunday_3_hour"], value[5])
        self._update_attribute(self.attridx["schedule_sunday_3_minute"], value[4])
        self._update_attribute(
            self.attridx["schedule_sunday_3_temperature"], value[3] / 2
        )
        self._update_attribute(self.attridx["schedule_sunday_4_hour"], value[2])
        self._update_attribute(self.attridx["schedule_sunday_4_minute"], value[1])
        self._update_attribute(
            self.attridx["schedule_sunday_4_temperature"], value[0] / 2
        )


class RtiUserInterface(TuyaUserInterfaceCluster):
    """HVAC User interface cluster for tuya electric heating thermostats."""

    _CHILD_LOCK_ATTR = RTI_CHILD_LOCK_ATTR


class RtiWindowDetection(LocalDataCluster, BinaryInput):
    """Binary cluster for the window detection function."""

    def __init__(self, *args, **kwargs):
        """Init."""
        super().__init__(*args, **kwargs)
        self.endpoint.device.RtiWindowDetection_bus.add_listener(self)

    def set_value(self, value):
        """Set value."""
        self._update_attribute(self.attridx["present_value"], value)


class RtiChildLock(CustomTuyaOnOff):
    """On/Off cluster for the child lock function of the electric heating thermostats."""

    def child_lock_change(self, value):
        """Child lock change."""
        self._update_attribute(self.attridx["on_off"], value)

    def map_attribute(self, attribute, value):
        """Map standardized attribute value to dict of manufacturer values."""
        if attribute == "on_off":
            return {RTI_CHILD_LOCK_ATTR: value}


class RtiValvePosition(LocalDataCluster, AnalogOutput):
    """Analog output for Valve State."""

    def __init__(self, *args, **kwargs):
        """Init."""
        super().__init__(*args, **kwargs)
        self.endpoint.device.RtiValvePosition_bus.add_listener(self)
        self._update_attribute(self.attridx["description"], "Valve State")
        self._update_attribute(self.attridx["max_present_value"], 100)
        self._update_attribute(self.attridx["min_present_value"], 0)
        self._update_attribute(self.attridx["resolution"], 1)
        self._update_attribute(self.attridx["application_type"], 4 << 16)
        self._update_attribute(self.attridx["engineering_units"], 98)

    def set_value(self, value):
        """Set value."""
        self._update_attribute(self.attridx["present_value"], value)


class RtiTempCalibration(LocalDataCluster, AnalogOutput):
    """Analog output for Temp Calibration."""

    def __init__(self, *args, **kwargs):
        """Init."""
        super().__init__(*args, **kwargs)
        self.endpoint.device.RtiTempCalibration_bus.add_listener(self)
        self._update_attribute(self.attridx["description"], "Temperature Calibration")
        self._update_attribute(self.attridx["max_present_value"], 20)
        self._update_attribute(self.attridx["min_present_value"], -20)
        self._update_attribute(self.attridx["resolution"], 1)
        self._update_attribute(self.attridx["application_type"], 13 << 16)
        self._update_attribute(self.attridx["engineering_units"], 62)

    def set_value(self, value):
        """Set value."""
        self._update_attribute(self.attridx["present_value"], value)

    def get_value(self):
        """Get value."""
        return self._attr_cache.get(self.attridx["present_value"])

    async def write_attributes(self, attributes, manufacturer=None):
        """Override the default Cluster write_attributes."""
        for attrid, value in attributes.items():
            if isinstance(attrid, str):
                attrid = self.attridx[attrid]
            if attrid not in self.attributes:
                self.error("%d is not a valid attribute id", attrid)
                continue
            self._update_attribute(attrid, value)

            await RtiManufClusterSelf[
                self.endpoint.device.ieee
            ].endpoint.tuya_manufacturer.write_attributes(
                {RTI_TEMP_CALIBRATION_ATTR: value},
                manufacturer=None,
            )
        return ([foundation.WriteAttributesStatusRecord(foundation.Status.SUCCESS)],)


class RtiBoostCountdown(LocalDataCluster, AnalogOutput):
    """Analog output for Boost Countdown time."""

    def __init__(self, *args, **kwargs):
        """Init."""
        super().__init__(*args, **kwargs)
        self.endpoint.device.RtiBoostCountdown_bus.add_listener(self)
        self._update_attribute(self.attridx["description"], "Boost Countdown")
        self._update_attribute(self.attridx["max_present_value"], 9999)
        self._update_attribute(self.attridx["min_present_value"], 0)
        self._update_attribute(self.attridx["resolution"], 1)
        self._update_attribute(self.attridx["application_type"], 12 << 16)
        self._update_attribute(self.attridx["engineering_units"], 72)

    def set_value(self, value):
        """Set value."""
        self._update_attribute(self.attridx["present_value"], value)


class RtiMinTemp(LocalDataCluster, AnalogOutput):
    """Analog output for Min Temperature."""

    def __init__(self, *args, **kwargs):
        """Init."""
        super().__init__(*args, **kwargs)
        self.endpoint.device.RtiMinTemp_bus.add_listener(self)
        self._update_attribute(self.attridx["description"], "Min Temperature")
        self._update_attribute(self.attridx["max_present_value"], 15)
        self._update_attribute(self.attridx["min_present_value"], 5)
        self._update_attribute(self.attridx["resolution"], 1)
        self._update_attribute(self.attridx["application_type"], 13 << 16)
        self._update_attribute(self.attridx["engineering_units"], 62)

    def set_value(self, value):
        """Set value."""
        self._update_attribute(self.attridx["present_value"], value)

    def get_value(self):
        """Get value."""
        return self._attr_cache.get(self.attridx["present_value"])

    async def write_attributes(self, attributes, manufacturer=None):
        """Override the default Cluster write_attributes."""
        for attrid, value in attributes.items():
            if isinstance(attrid, str):
                attrid = self.attridx[attrid]
            if attrid not in self.attributes:
                self.error("%d is not a valid attribute id", attrid)
                continue
            self._update_attribute(attrid, value)

            await RtiManufClusterSelf[
                self.endpoint.device.ieee
            ].endpoint.tuya_manufacturer.write_attributes(
                {RTI_MIN_TEMPERATURE_ATTR: value * 10},
                manufacturer=None,
            )
        return ([foundation.WriteAttributesStatusRecord(foundation.Status.SUCCESS)],)


class RtiMaxTemp(LocalDataCluster, AnalogOutput):
    """Analog output for Max Temperature."""

    def __init__(self, *args, **kwargs):
        """Init."""
        super().__init__(*args, **kwargs)
        self.endpoint.device.RtiMaxTemp_bus.add_listener(self)
        self._update_attribute(self.attridx["description"], "Max Temperature")
        self._update_attribute(self.attridx["max_present_value"], 35)
        self._update_attribute(self.attridx["min_present_value"], 15)
        self._update_attribute(self.attridx["resolution"], 1)
        self._update_attribute(self.attridx["application_type"], 13 << 16)
        self._update_attribute(self.attridx["engineering_units"], 62)

    def set_value(self, value):
        """Set value."""
        self._update_attribute(self.attridx["present_value"], value)

    def get_value(self):
        """Get value."""
        return self._attr_cache.get(self.attridx["present_value"])

    async def write_attributes(self, attributes, manufacturer=None):
        """Override the default Cluster write_attributes."""
        for attrid, value in attributes.items():
            if isinstance(attrid, str):
                attrid = self.attridx[attrid]
            if attrid not in self.attributes:
                self.error("%d is not a valid attribute id", attrid)
                continue
            self._update_attribute(attrid, value)

            await RtiManufClusterSelf[
                self.endpoint.device.ieee
            ].endpoint.tuya_manufacturer.write_attributes(
                {RTI_MAX_TEMPERATURE_ATTR: value * 10},
                manufacturer=None,
            )
        return ([foundation.WriteAttributesStatusRecord(foundation.Status.SUCCESS)],)


class Rti(TuyaThermostat):
    """Rti Thermostatic radiator valve."""

    def __init__(self, *args, **kwargs):
        """Init device."""
        self.thermostat_onoff_bus = Bus()
        self.RtiWindowDetection_bus = Bus()
        self.RtiValvePosition_bus = Bus()
        self.RtiTempCalibration_bus = Bus()
        self.RtiBoostCountdown_bus = Bus()
        self.RtiMinTemp_bus = Bus()
        self.RtiMaxTemp_bus = Bus()
        super().__init__(*args, **kwargs)

    signature = {
        #  endpoint=1 profile=260 device_type=81 device_version=0 input_clusters=[0, 4, 5, 61184]
        #  output_clusters=[10, 25]>
        MODELS_INFO: [
            ("_TZE200_a4bpgplm", "TS0601"),
        ],
        ENDPOINTS: {
            1: {
                PROFILE_ID: zha.PROFILE_ID,
                DEVICE_TYPE: zha.DeviceType.SMART_PLUG,
                INPUT_CLUSTERS: [
                    Basic.cluster_id,
                    Groups.cluster_id,
                    Scenes.cluster_id,
                    TuyaManufClusterAttributes.cluster_id,
                ],
                OUTPUT_CLUSTERS: [Time.cluster_id, Ota.cluster_id],
            }
        },
    }

    replacement = {
        ENDPOINTS: {
            1: {
                PROFILE_ID: zha.PROFILE_ID,
                DEVICE_TYPE: zha.DeviceType.THERMOSTAT,
                INPUT_CLUSTERS: [
                    Basic.cluster_id,
                    Groups.cluster_id,
                    Scenes.cluster_id,
                    RtiManufCluster,
                    RtiThermostat,
                    RtiUserInterface,
                    RtiWindowDetection,
                    TuyaPowerConfigurationCluster,
                ],
                OUTPUT_CLUSTERS: [Time.cluster_id, Ota.cluster_id],
            },
            2: {
                PROFILE_ID: zha.PROFILE_ID,
                DEVICE_TYPE: zha.DeviceType.ON_OFF_SWITCH,
                INPUT_CLUSTERS: [RtiChildLock],
                OUTPUT_CLUSTERS: [],
            },
            3: {
                PROFILE_ID: zha.PROFILE_ID,
                DEVICE_TYPE: zha.DeviceType.CONSUMPTION_AWARENESS_DEVICE,
                INPUT_CLUSTERS: [RtiValvePosition],
                OUTPUT_CLUSTERS: [],
            },
            4: {
                PROFILE_ID: zha.PROFILE_ID,
                DEVICE_TYPE: zha.DeviceType.CONSUMPTION_AWARENESS_DEVICE,
                INPUT_CLUSTERS: [RtiTempCalibration],
                OUTPUT_CLUSTERS: [],
            },
            5: {
                PROFILE_ID: zha.PROFILE_ID,
                DEVICE_TYPE: zha.DeviceType.CONSUMPTION_AWARENESS_DEVICE,
                INPUT_CLUSTERS: [RtiBoostCountdown],
                OUTPUT_CLUSTERS: [],
            },
            6: {
                PROFILE_ID: zha.PROFILE_ID,
                DEVICE_TYPE: zha.DeviceType.CONSUMPTION_AWARENESS_DEVICE,
                INPUT_CLUSTERS: [RtiMinTemp],
                OUTPUT_CLUSTERS: [],
            },
            7: {
                PROFILE_ID: zha.PROFILE_ID,
                DEVICE_TYPE: zha.DeviceType.CONSUMPTION_AWARENESS_DEVICE,
                INPUT_CLUSTERS: [RtiMaxTemp],
                OUTPUT_CLUSTERS: [],
            },
        }
    }
